
#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"
//#define F_CPU 16000000UL

#define VENDOR_RQ_GET_SOUND    0
#define VENDOR_RQ_SET_TH       1
char count=1;
uint16_t max=0;
uint16_t ov_counter=0;

uint16_t th;
char    is_up[2];
uint16_t peak[2];
uint16_t prev[2];
char sent;
char before;
uint16_t timediff;

char lock;

uint16_t val[2];
/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
/* ------------------------------------------------------------------------- */

char in_c(char pin)
{
    // read logic level of port-C's pin
    DDRC &= ~(1<<pin);
    return ((PINC & (1<<pin))>>pin);    
} 

void out_c(char pin, char val)
{
    // write logic level to port-C's pin
    DDRC |= 1<<pin;
    if(val)
        PORTC |= 1<<pin;
    else
        PORTC &= ~(1<<pin);
}

uint16_t read_adc(uint8_t channel)
{
    // read analog value from specified channel of port C
    ADMUX = 0x40;
    ADMUX |= channel;
    ADCSRA = 0xC6;
    while ((ADCSRA & (1<<ADSC)))
        ;
    return ADCL + ADCH*256;
}
/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */
void find_th()
{
	//start find threshold
	out_c(2,0);
	out_c(3,0);
	ov_counter=0;
	while(ov_counter<10)
		usbPoll();
	th=(read_adc(0)+read_adc(1))/2;
	while(ov_counter<25)
	{
		usbPoll();
		th=(th+read_adc(0)+read_adc(1))/3;
	}
	th+=150;
	out_c(2,1);
	out_c(3,1);
}

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
    usbRequest_t *rq = (void *)data;
    static uchar dataBuffer[1];  /* must stay valid when usbFunctionSetup returns */
	uint16_t x;
	if (rq->bRequest == VENDOR_RQ_GET_SOUND)
	{
		if(sent==0)
		{			
			//dataBuffer[0]=timediff%256;
			//dataBuffer[1]=timediff>>8;
			dataBuffer[0]=before;

			//dataBuffer[3]=peak[0]%256;
			//dataBuffer[4]=peak[0]>>8;

			//dataBuffer[5]=peak[1]%256;
			//dataBuffer[6]=peak[1]>>8;
			sent=1;		
		}
		else
		{
			dataBuffer[0]=2;
			//dataBuffer[0]=0;
			//dataBuffer[1]=0;
			//dataBuffer[2]=99;
		}		
		usbMsgPtr = dataBuffer;
		return 1;
	}
/*
	else if (rq->bRequest == VENDOR_RQ_SET_TH)
	{
		find_th();
		dataBuffer[0]=th%256;
		dataBuffer[1]=th>>8;
		return 2;
	}
*/
    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */
ISR(TIMER1_OVF_vect){
  	ov_counter++;
}

uint16_t waitfor(char n)
{	
	TCNT1=0;		
	ov_counter=0;
	while(ov_counter<1)
	{
		usbPoll();		
		val[n]=read_adc(n);
		if(val[n]>prev[n])
		{	
			is_up[n]=1;
		}
		else if(val[n]<prev[n])
		{
			if(is_up[n]==1)			
			{
				peak[n]=prev[n];				
				if(prev[n]>=th && prev[n]<peak[(n+1)%2])    //found a peak at second mic
				{										
					usbPoll();
					peak[n]=prev[n];
					is_up[n]=0;
					prev[n]=val[n];
					return TCNT1;
				}
			}
			is_up[n]=0;
		}
		prev[n]=val[n];
	}
	return 0;
}

int main(void)
{
    lock=0;
	th=550;
	is_up[0]=1;
	is_up[1]=1;
	peak[0]=150;
	peak[1]=150;
	prev[0]=150;
	prev[1]=150;

	before=0;
	timediff=0;
	sent=1;

	DDRC  = 0b00001100;  // PC0..PC2 = Output, PC3..PC5 = Input
    //PORTC |= 0b00001100;  // PC7..PC0 = 0
    //PORTC |= (1 << 3);   // Enable pull-up at PC3
    DDRD  = 0b00001000;
    PORTD = 0b00000000;

    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(300);
    usbDeviceConnect();

		
	count=0;
	TIMSK1=1;   //enable overflow interrupt
	TCCR1B=2;    //0=select no clock, 1= "1:1"(noPrescale) ,2="1:8",3="1:64",4="1:256",5="1:1024" 
	sei();
	
	find_th();   //threshold
	char i,t;    //t=turn
	PORTC |= 0b00001100;
	for(i=0;;usbPoll(),++i)
	{                /* main event loop */		
		t=i%2;		
		val[t]=read_adc(t);
		if(val[t]>prev[t])
		{	
			is_up[t]=1;
		}
		else if(val[t]<prev[t])
		{
			if(is_up[t]==1)			
			{	
				if(prev[t]>=th)    //start at t
				{					
					usbPoll();
					peak[t]=prev[t];					
					timediff=waitfor((t+1)%2);
					if(timediff!=0)
					{
						before=t;
						sent=0;
						out_c(t+2,1);
						out_c((t+1)%2+2,0);
						while(ov_counter<5)
							usbPoll();
					}
				}
			}
			is_up[t]=0;
		}
		prev[t]=val[t];
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
