import wx
import random
import usb

def find_board():
	boardx = None
	for bus in usb.busses():
		for dev in bus.devices:
			if dev.idVendor == 0x16c0 and dev.idProduct == 0x05dc:
				return dev
	return None

class Tetris(wx.Frame):
	def __init__(self, parent, id, title):
		wx.Frame.__init__(self, parent, id, title, size=(270, 570))

		self.statusbar = self.CreateStatusBar()
		self.statusbar.SetStatusText('Level : 1 | Point : 0')
		self.board = Board(self)
		self.board.SetFocus()
		self.board.start()

		self.Centre()
		self.Show(True)

class Board(wx.Panel):
	BoardWidth = 9
	BoardHeight = 22
	x=0
	y=0
	def __init__(self, *args, **kwargs):
		wx.Panel.__init__(self, *args, **kwargs)
		self.isStarted = False
		self.isPaused = False
		self.score = 0
		self.speed = 50
		self.level = 1
		self.Bind(wx.EVT_PAINT, self.OnPaint)
		self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
		self.timer1 = wx.Timer(self, 1)
		self.Bind(wx.EVT_TIMER, self.OnTimer, id=1)

		self.board = [0]*(self.BoardWidth*(self.BoardHeight + 3))

		#timer2 for input from controller
		if boardx!=None:
			self.timer2 = wx.Timer(self, 2)
			self.Bind(wx.EVT_TIMER, self.OnControl, id=2)
			#self.find_th()
			self.timer2.Start(10)
		else:
			print "board not found"

		self.mycar = 3
		self.create_mycar()
		
		self.color = random.randint(1, 7)
		self.enemyx = (random.randint(0, 2)) * 3
		self.enemyy = 18
		self.create_enemycar()
		
		self.Refresh()
		wx.MessageDialog(None, 'Are u Ready?', 'Start Game', wx.OK | wx.ICON_INFORMATION).ShowModal()
		self.timer1.Start(self.speed)

	def find_th(self):
		x= handle.controlMsg(req,1,2)
		print x[0]+x[1]*256
	
	def OnPaint(self, event=None):
		dc = wx.PaintDC(self)        

		size = self.GetClientSize()
		boardTop = size.GetHeight() - Board.BoardHeight * self.squareHeight()
        
		for i in range(Board.BoardHeight):
			for j in range(Board.BoardWidth):
				shape = self.shapeAt(j, Board.BoardHeight - i - 1)
				if shape != 0:
					self.drawSquare(dc,0 + j * self.squareWidth(),boardTop + i * self.squareHeight(), shape)

	def OnKeyDown(self, event):
		keycode = event.GetKeyCode()
		if keycode == wx.WXK_SPACE:
			self.pause()
			return

		elif keycode == ord('R') or keycode == ord('r'):
			self.timer1.Stop()
			self.isStarted = True
			self.isPaused = False
			self.score = 0
			self.speed = 50
			self.level = 1

			self.board = [0]*(self.BoardWidth*(self.BoardHeight + 3))

			self.mycar = 3
			self.create_mycar()
		
			self.color = random.randint(1, 7)
			self.enemyx = (random.randint(0, 2)) * 3
			self.enemyy = 18
			self.create_enemycar()
			self.GetParent().statusbar.SetStatusText('Level : '+str(self.level)+' | Point : '+str(self.score))
			self.Refresh()
			wx.MessageDialog(None, 'Are u Ready?', 'Start Game', wx.OK | wx.ICON_INFORMATION).ShowModal()
			self.timer1.Start(self.speed)
			return

		elif keycode == ord('T') or keycode == ord('t'):
			if self.isPaused:
				self.GetParent().statusbar.SetStatusText('Auto Thresholding')
				self.find_th()
			return

		if self.isPaused:
			return
		elif keycode == wx.WXK_LEFT:
			self.del_mycar()
			if self.mycar > 0:
				self.mycar -= 3
			self.create_mycar()
			self.Refresh()
		elif keycode == wx.WXK_RIGHT:
			self.del_mycar()
			if self.mycar < 6:
				self.mycar += 3
			self.create_mycar()
			self.Refresh()
		else:
			event.Skip()

	def OnControl(self, event):
		if not self.isPaused:
			x = handle.controlMsg(req,0,1)
			if x[0]==1:
				self.del_mycar()
				if self.mycar > 0:
					self.mycar -= 3					#the last bug
				self.create_mycar()
				self.Refresh()
			elif x[0]==0:
				self.del_mycar()
				if self.mycar < 6:
					self.mycar += 3					#the last bug
				self.create_mycar()
				self.Refresh()		
		#if x[2]!=99:		
		#	print x[1]*256+x[0],"--",x[2],'-                 -',x[4]*256+x[3],'**',x[6]*256+x[5]	

	def OnTimer(self, event):
		if event.GetId() == 1:
			if self.enemyy + 3 == 0:
				self.score += 10 * self.level
				if self.score == 30 * (self.level * (self.level + 1) / 2):
					self.speed -= 5
					if self.speed <= 0:
						self.speed = 5
					else:
						self.level += 1
				self.timer1.Start(self.speed)
				self.GetParent().statusbar.SetStatusText('Level : '+str(self.level)+' | Point : '+str(self.score))
				self.del_enemycar()
				self.Refresh()
				self.enemyx = (random.randint(0, 2)) * 3
				self.color = (random.randint(1,7))
				self.enemyy = 19
			elif self.enemyy <= 3 and self.enemyx == self.mycar:
				self.timer1.Stop()
				self.isStarted = False
				self.isPaused = True
				self.GetParent().statusbar.SetStatusText('Level : '+str(self.level)+' | Point : '+str(self.score))
				wx.MessageDialog(None, 'Press R to play again.', 'Game Over', wx.OK | wx.ICON_EXCLAMATION).ShowModal()
			else:
				self.del_enemycar()
				self.enemyy -= 1
				self.create_enemycar()
				self.Refresh()
		else:
			event.Skip()

	def drawSquare(self, dc, x, y, shape):
		colors = ['#000000', '#CC6666', '#66CC66', '#6666CC',
                  '#CCCC66', '#CC66CC', '#66CCCC', '#DAAA00']

		light = ['#000000', '#F89FAB', '#79FC79', '#7979FC', 
                 '#FCFC79', '#FC79FC', '#79FCFC', '#FCC600']

		dark = ['#000000', '#803C3B', '#3B803B', '#3B3B80', 
                 '#80803B', '#803B80', '#3B8080', '#806200']

		pen = wx.Pen(light[shape])
		pen.SetCap(wx.CAP_PROJECTING)
		dc.SetPen(pen)

		dc.DrawLine(x, y + self.squareHeight() - 1, x, y)
		dc.DrawLine(x, y, x + self.squareWidth() - 1, y)

		darkpen = wx.Pen(dark[shape])
		darkpen.SetCap(wx.CAP_PROJECTING)
		dc.SetPen(darkpen)

		dc.DrawLine(x + 1, y + self.squareHeight() - 1, x + self.squareWidth() - 1, y + self.squareHeight() - 1)
		dc.DrawLine(x + self.squareWidth() - 1, y + self.squareHeight() - 1, x + self.squareWidth() - 1, y + 1)

		dc.SetPen(wx.TRANSPARENT_PEN)
		dc.SetBrush(wx.Brush(colors[shape]))
		dc.DrawRectangle(x + 1, y + 1, self.squareWidth() - 2,
		self.squareHeight() - 2)

	def squareWidth(self):
		return self.GetClientSize().GetWidth() / Board.BoardWidth

	def squareHeight(self):
		return self.GetClientSize().GetHeight() / Board.BoardHeight

	def shapeAt(self, x, y):
		return self.board[(y * Board.BoardWidth) + x]

	def start(self):
		if self.isPaused:
			return
		self.isStarted = True
		self.timer1.Start(self.speed)

	def pause(self):
		if not self.isStarted:
			return
		self.isPaused = not self.isPaused
		if self.isPaused:
			self.timer1.Stop()
			self.GetParent().statusbar.SetStatusText('Paused')
		else:
			self.timer1.Start(self.speed)
			self.GetParent().statusbar.SetStatusText('Level : '+str(self.level)+' | Point : '+str(self.score))
		self.Refresh()

	def create_mycar(self):
		self.board[self.mycar                           : self.mycar + 3] = [3,3,3]
		self.board[1 * self.BoardWidth + self.mycar + 1 : 1 * self.BoardWidth + self.mycar + 2] = [3]
		self.board[2 * self.BoardWidth + self.mycar     : 2 * self.BoardWidth + self.mycar + 3] = [3,3,3]
		self.board[3 * self.BoardWidth + self.mycar + 1 : 3 * self.BoardWidth + self.mycar + 2] = [3]

	def del_mycar(self):
		self.board[self.mycar : self.mycar + 3] = [0,0,0]
		self.board[1 * self.BoardWidth + self.mycar + 1 : 1 * self.BoardWidth + self.mycar + 2] = [0]
		self.board[2 * self.BoardWidth + self.mycar : 2 * self.BoardWidth + self.mycar + 3] = [0,0,0]
		self.board[3 * self.BoardWidth + self.mycar + 1 : 3 * self.BoardWidth + self.mycar + 2] = [0]

	def create_enemycar(self):
		self.board[self.enemyy * self.BoardWidth + self.enemyx : self.enemyy * self.BoardWidth + self.enemyx + 3] = [self.color,self.color,self.color]
		self.board[(self.enemyy + 1) * self.BoardWidth + self.enemyx + 1 : (self.enemyy + 1) * self.BoardWidth + self.enemyx + 2] = [self.color]
		self.board[(self.enemyy + 2) * self.BoardWidth + self.enemyx : (self.enemyy + 2) * self.BoardWidth + self.enemyx + 3] = [self.color,self.color,self.color]
		self.board[(self.enemyy + 3) * self.BoardWidth + self.enemyx + 1 : (self.enemyy + 3) * self.BoardWidth + self.enemyx +2] = [self.color]

	def del_enemycar(self):
		self.board[self.enemyy * self.BoardWidth + self.enemyx : self.enemyy * self.BoardWidth + self.enemyx + 3] = [0,0,0]
		self.board[(self.enemyy + 1) * self.BoardWidth + self.enemyx + 1 : (self.enemyy + 1) * self.BoardWidth + self.enemyx + 2] = [0]
		self.board[(self.enemyy + 2) * self.BoardWidth + self.enemyx : (self.enemyy + 2) * self.BoardWidth + self.enemyx + 3] = [0,0,0]
		self.board[(self.enemyy + 3) * self.BoardWidth + self.enemyx + 1 : (self.enemyy + 3) * self.BoardWidth + self.enemyx +2]  = [0]

boardx=find_board()
if boardx!=None:
	handle=boardx.open()
req=usb.TYPE_VENDOR | usb.RECIP_DEVICE | usb.ENDPOINT_IN
app = wx.PySimpleApp(False)
Tetris(None, -1, 'Racing')
app.MainLoop()
